// Flasher -- based on tutorial from Adafruit
// https://learn.adafruit.com/multi-tasking-the-arduino-part-1


#ifndef _Flasher_h_
#define _Flasher_h_

#define FLASHER_VERSION "$Id: Flasher.h 2 2018-08-29 07:08:22Z cafelizardo $"

enum { DISABLED = -1, OFF=LOW, ON=HIGH } LED_STATE;

class Flasher
{
    protected:
    // configuration variables
    int ledPin;                      // number of the LED pin
    long onTime;                     // milliseconds of on-time
    long offTime;                    // milliseconds of off-time
    bool inverted;                   // invert output

    // state variables
    int ledState;                    // current value: DISABLED, OFF or ON
    unsigned long previousMillis;    // last time LED was last updated

    public:
    // constructor -- creates a Flasher object
    // and initialized the member variables and state
    Flasher(int pin = 0, long on = 0, long off = 0)
    {
        this->ledPin = pin;
        pinMode(ledPin, OUTPUT);

        this->onTime = on;
        this->offTime = off;
        this->inverted = false;
        this->ledState = OFF;
        this->previousMillis = 0;
    } // ctor

    // modifiers
    Flasher& setOnTime(long onTime) {
        this->onTime = onTime;
        return *this;
    }
    Flasher& setOffTime(long offTime) {
        this->offTime = offTime;
        return *this;
    }
    Flasher& setInvert(bool invert) {
        this->inverted = invert;
        return *this;
    }
    virtual Flasher& output(int value)
    {
        if (this->ledState == DISABLED) {
            digitalWrite(this->ledPin, this->inverted ? HIGH : LOW);
        } else {
            digitalWrite(this->ledPin, constrain(this->inverted ? 255 - value : value, 0, 255) );
        }
        return *this;
    }
    virtual Flasher& output()
    {
        //Serial.print("Flasher.output "); Serial.print(this->ledPin, DEC); Serial.println("");
        this->output(this->ledState);
        return *this;
    }
    virtual Flasher& disable() {
        //Serial.print("disabling "); Serial.print(this->ledPin, DEC); Serial.println("");
        this->ledState = DISABLED;
        this->output();
        return *this;
    }
    virtual Flasher& enable() {
        //Serial.print("enabling "); Serial.print(this->ledPin, DEC); Serial.println("");
        this->ledState = OFF;
        this->previousMillis = 0;
        this->output();
        return *this;
    }

    virtual Flasher& update()
    {
        if (this->ledState == DISABLED) return *this;

        //Serial.print("flasher update "); Serial.print(this->ledPin, DEC); Serial.println("");
        // check to see if it's tme to change the state of the LED
        unsigned long currentMillis = millis();

        bool toggle = false;
        if ( (this->ledState == ON) && (currentMillis - this->previousMillis >= this->onTime) )
            toggle = true;
        else if ( (this->ledState == OFF) && (currentMillis - this->previousMillis >= this->offTime) )
            toggle = true;

        if (toggle)
        {
            this->ledState = !this->ledState;
            this->output();
            this->previousMillis = currentMillis;
        }
        return *this;
    } // update

}; // class Flasher

#endif // _flasher_h_
